#region Copyright 2013 Inner Drive Technology

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using InnerDrive.Geography;

#endregion

namespace ExportFogBugz
{
	/// <summary>
	/// Exports FogBugz data to a .csv file
	/// </summary>
	/// <revision revisionDate="2013-08-10 15:51:53 -05:00" author="David Braverman" type="New" 
	/// location="IDTWHQ">IT-1/Bugz 3817</revision>
	public class Exporter
	{
		#region Constructor

		public Exporter()
		{
			TimeZoneFactory.Instance.LoadDefaultZones();
			_timeZone = TimeZoneFactory.Instance.FindZone("America/Chicago");
		}
		#endregion

		#region Public methods

		public void Execute(string projectName, string fileName)
		{
			Load(projectName);
			Export(fileName);
		}
		#endregion

		#region Private methods
		private static string Clean(string text)
		{
			text = text.Replace("\"", "'");
			text = text.Replace("&#x0D;", string.Empty);
			text = text.Replace("&lt;p&gt;", string.Empty);
			text = text.Replace("&lt;/p&gt;", Environment.NewLine);
			text = text.Replace("&#x20;", string.Empty);
			text = text.Replace("&#39;", "'");
			text = text.Replace("&lt;", "<");
			text = text.Replace("&gt;", ">");
			text = text.Replace("<li>", "*");
			text = text.Replace("</li>", string.Empty);
			text = text.Replace("<ul>", string.Empty);
			text = text.Replace("</ul>", string.Empty);
			text = text.Replace("&amp;", "&");
			text = text.Replace("&nbsp;", " ");
			return text;
		}

		private string FormatDate(DateTime? dateTime)
		{
			if (!dateTime.HasValue) return string.Empty;

			var date = dateTime.Value;
			var utc = new DateTimeOffset(date.Year, date.Month, date.Day, date.Hour, date.Minute, date.Second, TimeSpan.Zero);

			var localTime = _timeZone.ToLocalTime(utc);
			return localTime.ToString("yyyy-MM-dd HH:mm:ss");
		}

		private void Export(string fileName)
		{
			var lines = new StringBuilder();
			lines.Append(Header);
			lines.Append(Environment.NewLine);

			foreach (var item in _exportViews)
			{
				Write(lines, item);
				Console.WriteLine(item.Summary);
			}

			File.Delete(fileName);
			File.WriteAllText(fileName, lines.ToString());
		}

		private void Load(string projectName)
		{
			using (var db = new punzunsourceEntities())
			{
				_exportViews = db.ExportViews
					.Where(v => v.Project == projectName)
					.ToList();
			}
		}

		private static string Resolution(string text)
		{
			if (!text.Contains(" ")) return text + ",";
			var pieces = text.Split(' ');
			return pieces[0] + "," + pieces[1].Replace("(", string.Empty).Replace(")", string.Empty);
		}

		private static string Quote(string text)
		{
			return string.Format("\"{0}\"", Clean(text));
		}

		private void Write(StringBuilder sb, ExportView item)
		{
			sb.Append(item.FogBugzID);
			sb.Append(",");
			sb.Append(item.Category);
			sb.Append(",");
			sb.Append(item.Area);
			sb.Append(",");
			sb.Append(Quote(item.Summary));
			sb.Append(",");
			sb.Append(FormatDate(item.Opened));
			sb.Append(",");
			sb.Append(FormatDate(item.Resolved));
			sb.Append(",");
			sb.Append(FormatDate(item.Due));
			sb.Append(",");
			sb.Append(Resolution(item.Status));
			sb.Append(",");
			sb.Append(item.Priority);
			sb.Append(",");
			sb.Append(item.Fix_Version);
			sb.Append(",");
			sb.Append(Quote(item.Description));
			sb.Append(",");
			sb.Append(item.Original_Estimate);
			sb.Append(",");
			sb.Append(item.Remaining_Estimate);
			sb.Append(",");
			sb.Append(item.Logged);
			sb.Append(Environment.NewLine);
		}
		#endregion

		#region Private members

		private ICollection<ExportView> _exportViews;
		private readonly IdtTimeZone _timeZone;

		private const string Header = "BugzID,Category,Area,Summary,Opened,Resolved,Due,Status,Resolution," +
			"Priority,Version,Description,Original,Remaining,Logged Time";
		#endregion
	}
}